import 'dart:async';
import 'dart:convert';

import 'package:blusalt_identity/common/exception.dart';
import 'package:blusalt_identity/model/initialization_response.dart';
import 'package:blusalt_identity/network/network_client.dart';
import 'package:blusalt_identity/services.dart';

class BlusaltSDK {
  static final BlusaltSDK _instance = BlusaltSDK._internal();
  static ApiCredentials apiCredentials;
  static String _bundleID;
  static List<ServiceType> _services;

  static String get bundleID => _bundleID;
  static List<ServiceType> get services => _services;

  factory BlusaltSDK({ApiCredentials credentials}) {
    apiCredentials = credentials;
    return _instance;
  }

  static Future<Success<InitializationResponse>> initializeSDK(ApiCredentials credentials) async {
    BlusaltSDK(credentials: credentials);
    var client = new HttpClient();
    var result = await client.initializeSDK(credentials);
    Map<String, dynamic> map = jsonDecode(result.body);
    if (map['status_code'] == 200) {
      var response = InitializationResponse.fromJson(jsonDecode(result.body));
      _bundleID = response.result.bundleCallId;
      print(response);
      _services = _getEnableServices(response.result.services);
      _services.forEach((f){
        print(f.toString());
      });
      return Success(response.message, response);
    } else {
      throw NetworkResponseError(map['message'],
          code: map['status_code'], description: map['description']);
    }
  }

  BlusaltSDK._internal() {}

  static List<ServiceType> _getEnableServices(List<Services> services) {
    var list = List<ServiceType>();
    services.forEach((it) {
      list.add(IdentityService.getServiceFromLabel(it.serviceLabel));
    });
    return list;
  }

  static bool hasService(ServiceType serviceType){
    return _services.contains(serviceType);
  }
}
