import 'dart:convert';

import 'package:blusalt_identity/common/exception.dart';
import 'package:blusalt_identity/model/dl_response.dart';
import 'package:blusalt_identity/model/nin_response.dart';
import 'package:blusalt_identity/model/nip_response.dart';
import 'package:blusalt_identity/model/pvc_response.dart';
import 'package:blusalt_identity/model/request.dart';
import 'package:blusalt_identity/network/blusalt_sdk.dart';
import 'package:blusalt_identity/network/network_client.dart' as client;
import 'package:blusalt_identity/services.dart';
import 'package:http/http.dart';
import 'package:http_interceptor/http_interceptor.dart';

import 'model/bvn_response.dart';

final _netWorkClient = client.HttpClient();

Future<client.Resource<BVNResponse>> verifyBVN(BVNRequest bvnRequest) async {
  if (BlusaltSDK.apiCredentials == null) {
    throw InitializationException(
        "SDK initilization Failed. Please Initialize SDK ");
  }

  if(!BlusaltSDK.hasService(ServiceType.bvnVerification)){
    throw ServiceUnavailableException("BVN Verification Service is not available");
  }

  var result = await _netWorkClient.executeRequest(client.Request(
      url: "IdentityVerification/BVN",
      body: jsonEncode(bvnRequest),
      apiCredentials: BlusaltSDK.apiCredentials,
      requestMethod: Method.POST));

  var response = result as Response;
  Map<String, dynamic> map = jsonDecode(response.body);

  if (map['status_code'] == 200) {
    var bvnResponse = BVNResponse.fromJson(jsonDecode(response.body));
    return client.Success(bvnResponse.message, bvnResponse);
  } else {
    throw NetworkResponseError(map['message'],
        code: map['status_code'], description: map['description']);
  }
}

Future<client.Resource<NINResponse>> verifyNIN(NINRequest ninRequest) async {
  if (BlusaltSDK.apiCredentials == null) {
    throw InitializationException(
        "SDK initilization Failed. Please Initialize SDK ");
  }
  if(!BlusaltSDK.hasService(ServiceType.ninVerification)){
    throw ServiceUnavailableException("NIN Verification Service is not available");
  }

  var result = await _netWorkClient.executeRequest(client.Request(
      url: "IdentityVerification/NIN",
      body: jsonEncode(ninRequest),
      apiCredentials: BlusaltSDK.apiCredentials,
      requestMethod: Method.POST));

  var response = result as Response;
  Map<String, dynamic> map = jsonDecode(response.body);

  if (map['status_code'] == 200) {
    var ninResponse = NINResponse.fromJson(jsonDecode(response.body));
    return client.Success(ninResponse.message, ninResponse);
  } else {
    throw NetworkResponseError(map['message'],
        code: map['status_code'], description: map['description']);
  }
}

Future<client.Resource<NIPResponse>> verifyNIP(NIPRequest request) async {
  if (BlusaltSDK.apiCredentials == null) {
    throw InitializationException(
        "SDK initilization Failed. Please Initialize SDK ");
  }
  if(!BlusaltSDK.hasService(ServiceType.nipVerification)){
    throw ServiceUnavailableException("NIP Verification Service is not available");
  }

  var result = await _netWorkClient.executeRequest(client.Request(
      url: "IdentityVerification/NIP",
      body: jsonEncode(request),
      apiCredentials: BlusaltSDK.apiCredentials,
      requestMethod: Method.POST));
  var response = result as Response;
  Map<String, dynamic> map = jsonDecode(response.body);

  if (map['status_code'] == 200) {
    var nipResponse = NIPResponse.fromJson(jsonDecode(response.body));
    return client.Success(nipResponse.message, nipResponse);
  } else {
    throw NetworkResponseError(map['message'],
        code: map['status_code'], description: map['description']);
  }
}

Future<client.Resource<PVCResponse>> verifyPVC(PVCRequest request) async {
  if (BlusaltSDK.apiCredentials == null) {
    throw InitializationException(
        "SDK initilization Failed. Please Initialize SDK ");
  }
  if(!BlusaltSDK.hasService(ServiceType.pvcVerification)){
    throw ServiceUnavailableException("PVC Verification Service is not available");
  }
  var result = await _netWorkClient.executeRequest(client.Request(
      url: "IdentityVerification/PVC",
      body: jsonEncode(request),
      apiCredentials: BlusaltSDK.apiCredentials,
      requestMethod: Method.POST));
  var response = result as Response;
  Map<String, dynamic> map = jsonDecode(response.body);

  if (map['status_code'] == 200) {
    var pvcResponse = PVCResponse.fromJson(jsonDecode(response.body));
    return client.Success(pvcResponse.message, pvcResponse);
  } else {
    NetworkResponseError(map['message'],
        code: map['status_code'], description: map['description']);
  }
}

Future<client.Resource<DLResponse>> verifyDriversLicence(
    DLRequest request) async {
  if (BlusaltSDK.apiCredentials == null) {
    throw InitializationException(
        "SDK initilization Failed. Please Initialize SDK ");
  }
  if(!BlusaltSDK.hasService(ServiceType.dlVerification)){
    throw ServiceUnavailableException("Driver's Licence Verification Service is not available");
  }

  var result = _netWorkClient.executeRequest(client.Request(
      url: "IdentityVerification/DriverLicence",
      body: jsonEncode(request),
      apiCredentials: BlusaltSDK.apiCredentials,
      requestMethod: Method.POST));
  var response = result as Response;
  Map<String, dynamic> map = jsonDecode(response.body);

  if (map['status_code'] == 200) {
    var dlResponse = DLResponse.fromJson(jsonDecode(response.body));
    return client.Success(dlResponse.message, dlResponse);
  } else {
    throw NetworkResponseError(map['message'],
        code: map['status_code'], description: map['description']);
  }
}
